/* $Id: navierstokes.h,v 1.5 1999/08/25 19:37:43 knepley Exp $ */

/* Main include file for the Navier-Stokes flow test problem */

#ifndef __NAVIER_STOKES_H
#define __NAVIER_STOKES_H

#include "petscts.h"
#include "gsolver.h"
#include "src/snes/impls/ml/mlSNES.h"

extern int NAVIERSTOKES_COOKIE;
extern int NAVIERSTOKES_ComputeField;

enum {Stage_Setup, Stage_GridRefine, NumStages};
int *UserStages;

/* This structure holds information about physical properties */
typedef struct {
  double nu;      /* The kinematic viscosity */
  double fluidSG; /* The specific gravity of the fluid */
} PhysicalContext;

struct _NavierStokesContext {
  PETSCHEADER(int)
  Grid                   grid;              /* The problem grid */

  MeshBoundary2D         boundaryCtx;       /* Describes the domain boundary */
  MeshGeometryContext    geometryCtx;       /* Describes the domain geometry */
  PhysicalContext        physicalCtx;       /* Describes the fluid properties */

  int                    RofS;              /* The rate-of-strain operator */
  PetscTruth             useConvection;     /* Include convection in the fluid */
  PetscTruth             useML;             /* Flag for ML preconditioning */
  PetscTruth             useSchur;          /* Flag for Schur complement preconditioning */
  PetscTruth             useSecondary;      /* Flag for using a secondary nonlinear solver */

  GSNES                  snes;              /* The nonlinear solver context */
  GSNES                  snesSecondary;     /* The secondary nonlinear solver context */
  SLES                   sles;              /* The linear solver context */
  GMat                   A;                 /* The velocity system matrix */
  GVec                   u;                 /* The velocity solution vector */
  GVec                   p;                 /* The pressure solution vector */
  GVec                   f;                 /* The reduced rhs vector */
  GVec                   origF;             /* The original rhs vector */
  GVec                   uExact;            /* The exact velocity solution vector */
  GVec                   pExact;            /* The exact pressure solution vector */

  int                    dim;               /* The problem dimension */
  PetscTruth             linear;            /* Using linear elements (as opposed to quadratic) */
  int                    refineStep;        /* The step to start refining the mesh */
  int                    numLoops;          /* The number of systems to solve */

  PetscViewer            mathViewer;        /* Connection to Mathematica */
  GVecErrorKSPMonitorCtx kspMonitorCtx;     /* Specialized monitor for grid vector linear solves */
  GSNESErrorMonitorCtx   snesMonitorCtx;    /* Specialized monitor for grid vector nonlinear solves */
  PetscViewer            nonlinearViewer;   /* A viewer for analysis of the nonlinear iteration */
  PetscDrawHG            nonlinearHist;     /* A histogram of the number of nonlinear iterates for each time step */
  PetscViewer            secondaryViewer;   /* A viewer for analysis of the secondary nonlinear iteration */
  PetscDrawHG            secondaryHist;     /* A histogram of the number of secondary nonlinear iterates for each time step */
  PetscViewer            linearViewer;      /* A viewer for analysis of the linear iteration */
  PetscDrawHG            linearHist;        /* A histogram of the number of linear iterates for each Newton step */
  PetscViewer            totalLinearViewer; /* A viewer for analysis of the linear iteration */
  PetscDrawHG            totalLinearHist;   /* A histogram of the number of total linear iterates in a nonlinear solve */
  PetscViewer            innerLinearViewer; /* A viewer for analysis of preconditioner iterations */
  PetscDrawHG            innerLinearHist;   /* A histogram of the number of linear iterates in a solve with A */
  PetscViewer            schurLinearViewer; /* A viewer for analysis of Schur complement iterations */
  PetscDrawHG            schurLinearHist;   /* A histogram of the number of linear iterates in a solve with B^T A^{-1} B */
};
typedef struct _NavierStokesContext *NavierStokesContext;

/* Creation functions */
EXTERN int NavierStokesInitializePackage(char *);
EXTERN int NavierStokesContextCreate(MPI_Comm, NavierStokesContext *);
EXTERN int NavierStokesSetup(NavierStokesContext);
EXTERN int NavierStokesCreateGrid(NavierStokesContext);
EXTERN int NavierStokesRefineGrid(NavierStokesContext);
EXTERN int NavierStokesCreateStructures(NavierStokesContext);

/* Setup functions */
EXTERN int NavierStokesSetupGrid(NavierStokesContext);
EXTERN int NavierStokesSetupRhsFunction(Grid, NavierStokesContext);
EXTERN int NavierStokesSetupBC(Grid, NavierStokesContext);
EXTERN int NavierStokesSetupSNES(SNES, NavierStokesContext);
EXTERN int NavierStokesSetupKSP(KSP, NavierStokesContext);
EXTERN int NavierStokesSetupPC(PC, NavierStokesContext);
EXTERN int NavierStokesSetupStructures(NavierStokesContext);

/* Cleanup functions */
EXTERN int NavierStokesDestroyStructures(NavierStokesContext);
EXTERN int NavierStokesContextDestroy(NavierStokesContext);

/* Checking functions */
EXTERN int MatCheckSymmetry(Mat);
EXTERN int NavierStokesCheckSolution(NavierStokesContext, GVec, GVec, const char []);

/* Callback functions */
EXTERN int NavierStokesSolutionFunction(Vec, void *);
EXTERN int VelocitySolutionFunction(int, int, double *, double *, double *, PetscScalar *, void*);
EXTERN int PressureSolutionFunction(int, int, double *, double *, double *, PetscScalar *, void*);
EXTERN int VelocityRhsFunction(int, int, double *, double *, double *, PetscScalar *, void*);

/* Operator functions */
EXTERN int ConvectionOperator(int, int, double *, double *, double *, int, PetscScalar **, PetscScalar *, void *);
EXTERN int LinearConvectionOperator(Discretization, Discretization, int, int, int, int, int, double *,
                                    PetscScalar, PetscScalar *, PetscScalar *, void *);
EXTERN int RateOfStrainTensor(Discretization, Discretization, int, int, int, int, int, double *,
                              PetscScalar, PetscScalar *, PetscScalar *, void *);

/* Analysis functions */
EXTERN int NavierStokesCreateHists(NavierStokesContext);
EXTERN int NavierStokesResetHists(NavierStokesContext, PetscTruth);
EXTERN int NavierStokesDestroyHists(NavierStokesContext);
EXTERN int NavierStokesUpdateLinearHists(SLES, NavierStokesContext);
EXTERN int NavierStokesUpdateGSNES(GSNES, int);
EXTERN int NavierStokesAnalyzeSolvers(NavierStokesContext);

/* Solver functions */
EXTERN int NavierStokesSolve(NavierStokesContext, GVec, GVec, int *);
EXTERN int NavierStokesComputeFlowField(NavierStokesContext);

#endif /* __NAVIER_STOKES_H */
